@@ -1,4 +1,4 @@
-/**
+/*
  * Licensed to the Apache Software Foundation (ASF) under one
  * or more contributor license agreements.  See the NOTICE file
  * distributed with this work for additional information
@@ -19,6 +19,7 @@
 package org.apache.bookkeeper.bookie;
 
 import static org.apache.bookkeeper.meta.MetadataDrivers.runFunctionWithLedgerManagerFactory;
+
 import com.google.common.annotations.VisibleForTesting;
 import java.io.File;
 import java.io.IOException;
@@ -38,6 +39,7 @@ import java.util.HashMap;
 import java.util.List;
 import java.util.Map;
 import java.util.stream.Collectors;
+import org.apache.bookkeeper.bookie.storage.EntryLogger;
 import org.apache.bookkeeper.client.LedgerEntry;
 import org.apache.bookkeeper.client.api.LedgerMetadata;
 import org.apache.bookkeeper.common.annotation.InterfaceAudience.Private;
@@ -158,7 +160,7 @@ public class BookieShell implements Tool {
     static final String CMD_REBUILD_DB_LEDGERS_INDEX = "rebuild-db-ledgers-index";
     static final String CMD_CHECK_DB_LEDGERS_INDEX = "check-db-ledgers-index";
     static final String CMD_REGENERATE_INTERLEAVED_STORAGE_INDEX_FILE = "regenerate-interleaved-storage-index-file";
-    static final String CMD_QUERY_AUTORECOVERY_STATUS = "queryrecoverystatus";
+    static final String CMD_QUERY_AUTORECOVERY_STATUS = "queryautorecoverystatus";
 
     // cookie commands
     static final String CMD_CREATE_COOKIE = "cookie_create";
@@ -803,6 +805,7 @@ public class BookieShell implements Tool {
             opts.addOption("l", "ledgerid", true, "Ledger ID");
             opts.addOption("dumptofile", true, "Dump metadata for ledger, to a file");
             opts.addOption("restorefromfile", true, "Restore metadata for ledger, from a file");
+            opts.addOption("update", false, "Update metadata if ledger already exist");
         }
 
         @Override
@@ -812,7 +815,7 @@ public class BookieShell implements Tool {
                 System.err.println("Must specify a ledger id");
                 return -1;
             }
-            if (cmdLine.hasOption("dumptofile") && cmdLine.hasOption("restorefromefile")) {
+            if (cmdLine.hasOption("dumptofile") && cmdLine.hasOption("restorefromfile")) {
                 System.err.println("Only one of --dumptofile and --restorefromfile can be specified");
                 return -2;
             }
@@ -825,6 +828,7 @@ public class BookieShell implements Tool {
             if (cmdLine.hasOption("restorefromfile")) {
                 flag.restoreFromFile(cmdLine.getOptionValue("restorefromfile"));
             }
+            flag.update(cmdLine.hasOption("update"));
 
             LedgerMetaDataCommand cmd = new LedgerMetaDataCommand(ledgerIdFormatter);
             cmd.apply(bkConf, flag);
@@ -838,7 +842,7 @@ public class BookieShell implements Tool {
 
         @Override
         String getUsage() {
-            return "ledgermetadata -ledgerid <ledgerid> [--dump-to-file FILENAME|--restore-from-file FILENAME]";
+            return "ledgermetadata -ledgerid <ledgerid> [--dumptofile FILENAME|--restorefromfile FILENAME]";
         }
 
         @Override
@@ -2413,7 +2417,7 @@ public class BookieShell implements Tool {
             if (cmdLine.hasOption(CONF_OPT)) {
                 String val = cmdLine.getOptionValue(CONF_OPT);
                 conf.addConfiguration(new PropertiesConfiguration(
-                    new File(val).toURI().toURL()));
+                        new File(val).toURI().toURL()));
             }
             shell.setConf(conf);
 
@@ -2424,7 +2428,9 @@ public class BookieShell implements Tool {
             } else {
                 shell.ledgerIdFormatter = LedgerIdFormatter.newLedgerIdFormatter(shell.bkConf);
             }
-            LOG.debug("Using ledgerIdFormatter {}", shell.ledgerIdFormatter.getClass());
+            if (LOG.isDebugEnabled()) {
+                LOG.debug("Using ledgerIdFormatter {}", shell.ledgerIdFormatter.getClass());
+            }
 
             // entry format
             if (cmdLine.hasOption(ENTRY_FORMATTER_OPT)) {
@@ -2433,7 +2439,9 @@ public class BookieShell implements Tool {
             } else {
                 shell.entryFormatter = EntryFormatter.newEntryFormatter(shell.bkConf);
             }
-            LOG.debug("Using entry formatter {}", shell.entryFormatter.getClass());
+            if (LOG.isDebugEnabled()) {
+                LOG.debug("Using entry formatter {}", shell.entryFormatter.getClass());
+            }
 
             res = shell.run(cmdLine.getArgs());
         } catch (Throwable e) {
@@ -2446,7 +2454,7 @@ public class BookieShell implements Tool {
     private synchronized void initEntryLogger() throws IOException {
         if (null == entryLogger) {
             // provide read only entry logger
-            entryLogger = new ReadOnlyEntryLogger(bkConf);
+            entryLogger = new ReadOnlyDefaultEntryLogger(bkConf);
         }
     }
 
