@@ -19,7 +19,6 @@
 package org.apache.bookkeeper.bookie;
 
 import java.util.concurrent.Future;
-
 /**
  * State management of Bookie, including register, turn bookie to w/r mode.
  */
@@ -49,11 +48,6 @@ public interface StateManager extends AutoCloseable {
      */
     boolean isReadOnly();
 
-    /**
-     * Check is forceReadOnly.
-     */
-    boolean isForceReadOnly();
-
     /**
      * Check is Running.
      */
