@@ -1,4 +1,4 @@
-/**
+/*
  *
  * Licensed to the Apache Software Foundation (ASF) under one
  * or more contributor license agreements.  See the NOTICE file
@@ -27,13 +27,12 @@ import com.google.common.annotations.VisibleForTesting;
 import com.google.common.collect.ImmutableMap;
 import com.google.common.collect.Lists;
 import com.google.protobuf.ByteString;
-
 import edu.umd.cs.findbugs.annotations.SuppressFBWarnings;
 import io.netty.buffer.ByteBuf;
 import io.netty.buffer.ByteBufAllocator;
 import io.netty.buffer.Unpooled;
+import io.netty.util.ReferenceCountUtil;
 import io.netty.util.concurrent.DefaultThreadFactory;
-
 import java.io.File;
 import java.io.IOException;
 import java.util.Collections;
@@ -49,7 +48,6 @@ import java.util.concurrent.TimeUnit;
 import java.util.concurrent.atomic.AtomicBoolean;
 import java.util.concurrent.locks.ReentrantLock;
 import java.util.concurrent.locks.StampedLock;
-
 import org.apache.bookkeeper.bookie.Bookie;
 import org.apache.bookkeeper.bookie.Bookie.NoEntryException;
 import org.apache.bookkeeper.bookie.BookieException;
@@ -59,7 +57,6 @@ import org.apache.bookkeeper.bookie.CheckpointSource.Checkpoint;
 import org.apache.bookkeeper.bookie.Checkpointer;
 import org.apache.bookkeeper.bookie.CompactableLedgerStorage;
 import org.apache.bookkeeper.bookie.EntryLocation;
-import org.apache.bookkeeper.bookie.EntryLogger;
 import org.apache.bookkeeper.bookie.GarbageCollectionStatus;
 import org.apache.bookkeeper.bookie.GarbageCollectorThread;
 import org.apache.bookkeeper.bookie.LastAddConfirmedUpdateNotification;
@@ -68,6 +65,7 @@ import org.apache.bookkeeper.bookie.LedgerDirsManager;
 import org.apache.bookkeeper.bookie.LedgerDirsManager.LedgerDirsListener;
 import org.apache.bookkeeper.bookie.LedgerEntryPage;
 import org.apache.bookkeeper.bookie.StateManager;
+import org.apache.bookkeeper.bookie.storage.EntryLogger;
 import org.apache.bookkeeper.bookie.storage.ldb.DbLedgerStorageDataFormats.LedgerData;
 import org.apache.bookkeeper.bookie.storage.ldb.KeyValueStorage.Batch;
 import org.apache.bookkeeper.common.util.Watcher;
@@ -80,6 +78,7 @@ import org.apache.bookkeeper.stats.StatsLogger;
 import org.apache.bookkeeper.stats.ThreadRegistry;
 import org.apache.bookkeeper.util.MathUtils;
 import org.apache.bookkeeper.util.collections.ConcurrentLongHashMap;
+import org.apache.commons.collections4.CollectionUtils;
 import org.apache.commons.lang.mutable.MutableLong;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
@@ -143,25 +142,34 @@ public class SingleDirectoryDbLedgerStorage implements CompactableLedgerStorage
     private final long maxReadAheadBytesSize;
 
     private final Counter flushExecutorTime;
-    private final boolean singleLedgerDirs;
 
     public SingleDirectoryDbLedgerStorage(ServerConfiguration conf, LedgerManager ledgerManager,
-            LedgerDirsManager ledgerDirsManager, LedgerDirsManager indexDirsManager, StatsLogger statsLogger,
-            ByteBufAllocator allocator, ScheduledExecutorService gcExecutor, long writeCacheSize, long readCacheSize,
-            int readAheadCacheBatchSize) throws IOException {
+                                          LedgerDirsManager ledgerDirsManager, LedgerDirsManager indexDirsManager,
+                                          EntryLogger entryLogger, StatsLogger statsLogger, ByteBufAllocator allocator,
+                                          long writeCacheSize, long readCacheSize, int readAheadCacheBatchSize)
+            throws IOException {
         checkArgument(ledgerDirsManager.getAllLedgerDirs().size() == 1,
                 "Db implementation only allows for one storage dir");
 
-        String baseDir = ledgerDirsManager.getAllLedgerDirs().get(0).toString();
-        log.info("Creating single directory db ledger storage on {}", baseDir);
+        String ledgerBaseDir = ledgerDirsManager.getAllLedgerDirs().get(0).getPath();
+        // indexBaseDir default use ledgerBaseDir
+        String indexBaseDir = ledgerBaseDir;
+        if (CollectionUtils.isEmpty(indexDirsManager.getAllLedgerDirs())) {
+            log.info("indexDir is not specified, use default, creating single directory db ledger storage on {}",
+                    indexBaseDir);
+        } else {
+            // if indexDir is specified, set new value
+            indexBaseDir = indexDirsManager.getAllLedgerDirs().get(0).getPath();
+            log.info("indexDir is specified, creating single directory db ledger storage on {}", indexBaseDir);
+        }
 
-        StatsLogger ledgerDirStatsLogger = statsLogger.scopeLabel("ledgerDir",
-                ledgerDirsManager.getAllLedgerDirs().get(0).getPath());
+        StatsLogger ledgerIndexDirStatsLogger = statsLogger
+                .scopeLabel("ledgerDir", ledgerBaseDir)
+                .scopeLabel("indexDir", indexBaseDir);
 
         this.writeCacheMaxSize = writeCacheSize;
         this.writeCache = new WriteCache(allocator, writeCacheMaxSize / 2);
         this.writeCacheBeingFlushed = new WriteCache(allocator, writeCacheMaxSize / 2);
-        this.singleLedgerDirs = conf.getLedgerDirs().length == 1;
 
         readCacheMaxSize = readCacheSize;
         this.readAheadCacheBatchSize = readAheadCacheBatchSize;
@@ -175,9 +183,10 @@ public class SingleDirectoryDbLedgerStorage implements CompactableLedgerStorage
 
         readCache = new ReadCache(allocator, readCacheMaxSize);
 
-        ledgerIndex = new LedgerMetadataIndex(conf, KeyValueStorageRocksDB.factory, baseDir, ledgerDirStatsLogger);
+        ledgerIndex = new LedgerMetadataIndex(conf,
+                KeyValueStorageRocksDB.factory, indexBaseDir, ledgerIndexDirStatsLogger);
         entryLocationIndex = new EntryLocationIndex(conf,
-                KeyValueStorageRocksDB.factory, baseDir, ledgerDirStatsLogger);
+                KeyValueStorageRocksDB.factory, indexBaseDir, ledgerIndexDirStatsLogger);
 
         transientLedgerInfoCache = ConcurrentLongHashMap.<TransientLedgerInfo>newBuilder()
                 .expectedItems(16 * 1024)
@@ -187,27 +196,31 @@ public class SingleDirectoryDbLedgerStorage implements CompactableLedgerStorage
                 TransientLedgerInfo.LEDGER_INFO_CACHING_TIME_MINUTES,
                 TransientLedgerInfo.LEDGER_INFO_CACHING_TIME_MINUTES, TimeUnit.MINUTES);
 
-        entryLogger = new EntryLogger(conf, ledgerDirsManager, null, statsLogger, allocator);
-        gcThread = new GarbageCollectorThread(conf, ledgerManager, ledgerDirsManager, this, statsLogger);
+        this.entryLogger = entryLogger;
+        gcThread = new GarbageCollectorThread(conf,
+                ledgerManager, ledgerDirsManager, this, entryLogger, ledgerIndexDirStatsLogger);
 
         dbLedgerStorageStats = new DbLedgerStorageStats(
-                ledgerDirStatsLogger,
+            ledgerIndexDirStatsLogger,
             () -> writeCache.size() + writeCacheBeingFlushed.size(),
             () -> writeCache.count() + writeCacheBeingFlushed.count(),
             () -> readCache.size(),
             () -> readCache.count()
         );
 
-        flushExecutorTime = ledgerDirStatsLogger.getThreadScopedCounter("db-storage-thread-time");
+        flushExecutorTime = ledgerIndexDirStatsLogger.getThreadScopedCounter("db-storage-thread-time");
 
         executor.submit(() -> {
             ThreadRegistry.register(dbStoragerExecutorName, 0);
             // ensure the metric gets registered on start-up as this thread only executes
             // when the write cache is full which may not happen or not for a long time
-            flushExecutorTime.add(0);
+            flushExecutorTime.addLatency(0, TimeUnit.NANOSECONDS);
         });
 
         ledgerDirsManager.addLedgerDirsListener(getLedgerDirsListener());
+        if (!ledgerBaseDir.equals(indexBaseDir)) {
+            indexDirsManager.addLedgerDirsListener(getLedgerDirsListener());
+        }
     }
 
     @Override
@@ -252,7 +265,7 @@ public class SingleDirectoryDbLedgerStorage implements CompactableLedgerStorage
     }
 
     @Override
-    public void forceGC(boolean forceMajor, boolean forceMinor) {
+    public void forceGC(Boolean forceMajor, Boolean forceMinor) {
         gcThread.enableForceGC(forceMajor, forceMinor);
     }
 
@@ -261,13 +274,65 @@ public class SingleDirectoryDbLedgerStorage implements CompactableLedgerStorage
         return gcThread.isInForceGC();
     }
 
+    public void suspendMinorGC() {
+        gcThread.suspendMinorGC();
+    }
+
+    public void suspendMajorGC() {
+        gcThread.suspendMajorGC();
+    }
+
+    public void resumeMinorGC() {
+        gcThread.resumeMinorGC();
+    }
+
+    public void resumeMajorGC() {
+        gcThread.resumeMajorGC();
+    }
+
+    public boolean isMajorGcSuspended() {
+        return gcThread.isMajorGcSuspend();
+    }
+
+    public boolean isMinorGcSuspended() {
+        return gcThread.isMinorGcSuspend();
+    }
+
+    @Override
+    public void entryLocationCompact() {
+        if (entryLocationIndex.isCompacting()) {
+            // RocksDB already running compact.
+            return;
+        }
+        cleanupExecutor.execute(() -> {
+            // There can only be one single cleanup task running because the cleanupExecutor
+            // is single-threaded
+            try {
+                log.info("Trigger entry location index RocksDB compact.");
+                entryLocationIndex.compact();
+            } catch (Throwable t) {
+                log.warn("Failed to trigger entry location index RocksDB compact", t);
+            }
+        });
+    }
+
+    @Override
+    public boolean isEntryLocationCompacting() {
+        return entryLocationIndex.isCompacting();
+    }
+
+    @Override
+    public List<String> getEntryLocationDBPath() {
+        return Lists.newArrayList(entryLocationIndex.getEntryLocationDBPath());
+    }
+
     @Override
     public void shutdown() throws InterruptedException {
         try {
             flush();
 
             gcThread.shutdown();
-            entryLogger.shutdown();
+            entryLogger.close();
 
             cleanupExecutor.shutdown();
             cleanupExecutor.awaitTermination(1, TimeUnit.SECONDS);
@@ -452,7 +517,7 @@ public class SingleDirectoryDbLedgerStorage implements CompactableLedgerStorage
                         } catch (IOException e) {
                             log.error("Error during flush", e);
                         } finally {
-                            flushExecutorTime.add(MathUtils.elapsedNanos(startTime));
+                            flushExecutorTime.addLatency(MathUtils.elapsedNanos(startTime), TimeUnit.NANOSECONDS);
                         }
                     });
             }
@@ -559,14 +624,16 @@ public class SingleDirectoryDbLedgerStorage implements CompactableLedgerStorage
                 throw new NoEntryException(ledgerId, entryId);
             }
         } finally {
-            dbLedgerStorageStats.getReadFromLocationIndexTime().add(MathUtils.elapsedNanos(locationIndexStartNano));
+            dbLedgerStorageStats.getReadFromLocationIndexTime().addLatency(
+                    MathUtils.elapsedNanos(locationIndexStartNano), TimeUnit.NANOSECONDS);
         }
 
         long readEntryStartNano = MathUtils.nowInNano();
         try {
             entry = entryLogger.readEntry(ledgerId, entryId, entryLocation);
         } finally {
-            dbLedgerStorageStats.getReadFromEntryLogTime().add(MathUtils.elapsedNanos(readEntryStartNano));
+            dbLedgerStorageStats.getReadFromEntryLogTime().addLatency(
+                    MathUtils.elapsedNanos(readEntryStartNano), TimeUnit.NANOSECONDS);
         }
 
         readCache.put(ledgerId, entryId, entry);
@@ -591,8 +658,8 @@ public class SingleDirectoryDbLedgerStorage implements CompactableLedgerStorage
             while (count < readAheadCacheBatchSize
                     && size < maxReadAheadBytesSize
                     && currentEntryLogId == firstEntryLogId) {
-                ByteBuf entry = entryLogger.internalReadEntry(orginalLedgerId, firstEntryId, currentEntryLocation,
-                        false /* validateEntry */);
+                ByteBuf entry = entryLogger.readEntry(orginalLedgerId,
+                        firstEntryId, currentEntryLocation);
 
                 try {
                     long currentEntryLedgerId = entry.getLong(0);
@@ -613,7 +680,7 @@ public class SingleDirectoryDbLedgerStorage implements CompactableLedgerStorage
                     currentEntryLocation += 4 + entry.readableBytes();
                     currentEntryLogId = currentEntryLocation >> 32;
                 } finally {
-                    entry.release();
+                    ReferenceCountUtil.release(entry);
                 }
             }
         } catch (Exception e) {
@@ -623,7 +690,8 @@ public class SingleDirectoryDbLedgerStorage implements CompactableLedgerStorage
         } finally {
             dbLedgerStorageStats.getReadAheadBatchCountStats().registerSuccessfulValue(count);
             dbLedgerStorageStats.getReadAheadBatchSizeStats().registerSuccessfulValue(size);
-            dbLedgerStorageStats.getReadAheadTime().add(MathUtils.elapsedNanos(readAheadStartNano));
+            dbLedgerStorageStats.getReadAheadTime().addLatency(
+                    MathUtils.elapsedNanos(readAheadStartNano), TimeUnit.NANOSECONDS);
         }
     }
 
@@ -678,11 +746,13 @@ public class SingleDirectoryDbLedgerStorage implements CompactableLedgerStorage
         }
 
         long entryLocation = entryLocationIndex.getLocation(ledgerId, lastEntryId);
-        dbLedgerStorageStats.getReadFromLocationIndexTime().add(MathUtils.elapsedNanos(locationIndexStartNano));
+        dbLedgerStorageStats.getReadFromLocationIndexTime().addLatency(
+                MathUtils.elapsedNanos(locationIndexStartNano), TimeUnit.NANOSECONDS);
 
         long readEntryStartNano = MathUtils.nowInNano();
         ByteBuf content = entryLogger.readEntry(ledgerId, lastEntryId, entryLocation);
-        dbLedgerStorageStats.getReadFromEntryLogTime().add(MathUtils.elapsedNanos(readEntryStartNano));
+        dbLedgerStorageStats.getReadFromEntryLogTime().addLatency(
+                MathUtils.elapsedNanos(readEntryStartNano), TimeUnit.NANOSECONDS);
         return content;
     }
 
@@ -733,7 +803,7 @@ public class SingleDirectoryDbLedgerStorage implements CompactableLedgerStorage
 
             Batch batch = entryLocationIndex.newBatch();
             writeCacheBeingFlushed.forEach((ledgerId, entryId, entry) -> {
-                long location = entryLogger.addEntry(ledgerId, entry, true);
+                long location = entryLogger.addEntry(ledgerId, entry);
                 entryLocationIndex.addLocation(batch, ledgerId, entryId, location);
             });
 
@@ -754,6 +824,21 @@ public class SingleDirectoryDbLedgerStorage implements CompactableLedgerStorage
             ledgerIndex.flush();
             recordSuccessfulEvent(dbLedgerStorageStats.getFlushLedgerIndexStats(), ledgerIndexStartTime);
 
+            cleanupExecutor.execute(() -> {
+                // There can only be one single cleanup task running because the cleanupExecutor
+                // is single-threaded
+                try {
+                    if (log.isDebugEnabled()) {
+                        log.debug("Removing deleted ledgers from db indexes");
+                    }
+
+                    entryLocationIndex.removeOffsetFromDeletedLedgers();
+                    ledgerIndex.removeDeletedLedgers();
+                } catch (Throwable t) {
+                    log.warn("Failed to cleanup db indexes", t);
+                }
+            });
+
             lastCheckpoint = thisCheckpoint;
 
             // Discard all the entry from the write cache, since they're now persisted
@@ -774,21 +859,6 @@ public class SingleDirectoryDbLedgerStorage implements CompactableLedgerStorage
             throw e;
         } finally {
             try {
-                cleanupExecutor.execute(() -> {
-                    // There can only be one single cleanup task running because the cleanupExecutor
-                    // is single-threaded
-                    try {
-                        if (log.isDebugEnabled()) {
-                            log.debug("Removing deleted ledgers from db indexes");
-                        }
-
-                        entryLocationIndex.removeOffsetFromDeletedLedgers();
-                        ledgerIndex.removeDeletedLedgers();
-                    } catch (Throwable t) {
-                        log.warn("Failed to cleanup db indexes", t);
-                    }
-                });
-
                 isFlushOngoing.set(false);
             } finally {
                 flushMutex.unlock();
@@ -823,9 +893,7 @@ public class SingleDirectoryDbLedgerStorage implements CompactableLedgerStorage
     public void flush() throws IOException {
         Checkpoint cp = checkpointSource.newCheckpoint();
         checkpoint(cp);
-        if (singleLedgerDirs) {
-            checkpointSource.checkpointComplete(cp, true);
-        }
+        checkpointSource.checkpointComplete(cp, true);
     }
 
     @Override
@@ -863,35 +931,14 @@ public class SingleDirectoryDbLedgerStorage implements CompactableLedgerStorage
 
     @Override
     public void updateEntriesLocations(Iterable<EntryLocation> locations) throws IOException {
-        // Before updating the DB with the new location for the compacted entries, we need to
-        // make sure that there is no ongoing flush() operation.
-        // If there were a flush, we could have the following situation, which is highly
-        // unlikely though possible:
-        // 1. Flush operation has written the write-cache content into entry-log files
-        // 2. The DB location index is not yet updated
-        // 3. Compaction is triggered and starts compacting some of the recent files
-        // 4. Compaction will write the "new location" into the DB
-        // 5. The pending flush() will overwrite the DB with the "old location", pointing
-        //    to a file that no longer exists
-        //
-        // To avoid this race condition, we need that all the entries that are potentially
-        // included in the compaction round to have all the indexes already flushed into
-        // the DB.
-        // The easiest lightweight way to achieve this is to wait for any pending
-        // flush operation to be completed before updating the index with the compacted
-        // entries, by blocking on the flushMutex.
-        flushMutex.lock();
-        flushMutex.unlock();
+        // Trigger a flush to have all the entries being compacted in the db storage
+        flush();
 
-        // We don't need to keep the flush mutex locked here while updating the DB.
-        // It's fine to have a concurrent flush operation at this point, because we
-        // know that none of the entries being flushed was included in the compaction
-        // round that we are dealing with.
         entryLocationIndex.updateLocations(locations);
     }
 
-    @Override
-    public EntryLogger getEntryLogger() {
+    @VisibleForTesting
+    EntryLogger getEntryLogger() {
         return entryLogger;
     }
 
@@ -908,7 +955,7 @@ public class SingleDirectoryDbLedgerStorage implements CompactableLedgerStorage
                 lac = bb.readLong();
                 lac = getOrAddLedgerInfo(ledgerId).setLastAddConfirmed(lac);
             } finally {
-                bb.release();
+                ReferenceCountUtil.release(bb);
             }
         }
         return lac;
@@ -955,13 +1002,11 @@ public class SingleDirectoryDbLedgerStorage implements CompactableLedgerStorage
             }
             return null;
         }
-        if (ledgerData.hasExplicitLac()) {
-            if (log.isDebugEnabled()) {
-                log.debug("getExplicitLac ledger {} returned from LedgerData", ledgerId);
-            }
-            ByteString persistedLac = ledgerData.getExplicitLac();
-            ledgerInfo.setExplicitLac(Unpooled.wrappedBuffer(persistedLac.toByteArray()));
+        if (log.isDebugEnabled()) {
+            log.debug("getExplicitLac ledger {} returned from LedgerData", ledgerId);
         }
+        ByteString persistedLac = ledgerData.getExplicitLac();
+        ledgerInfo.setExplicitLac(Unpooled.wrappedBuffer(persistedLac.toByteArray()));
         return ledgerInfo.getExplicitLac();
     }
 
