@@ -1,4 +1,4 @@
-/**
+/*
  * Licensed to the Apache Software Foundation (ASF) under one
  * or more contributor license agreements.  See the NOTICE file
  * distributed with this work for additional information
@@ -24,7 +24,6 @@ import com.google.common.base.Joiner;
 import com.google.protobuf.InvalidProtocolBufferException;
 import com.google.protobuf.TextFormat;
 import com.google.protobuf.TextFormat.ParseException;
-
 import java.net.UnknownHostException;
 import java.util.Arrays;
 import java.util.Collection;
@@ -41,7 +40,6 @@ import java.util.concurrent.CountDownLatch;
 import java.util.function.Predicate;
 import java.util.regex.Matcher;
 import java.util.regex.Pattern;
-
 import org.apache.bookkeeper.common.concurrent.FutureUtils;
 import org.apache.bookkeeper.conf.AbstractConfiguration;
 import org.apache.bookkeeper.meta.zk.ZKMetadataDriverBase;
@@ -815,7 +813,9 @@ public class ZkLedgerUnderreplicationManager implements LedgerUnderreplicationMa
 
     @Override
     public boolean initializeLostBookieRecoveryDelay(int lostBookieRecoveryDelay) throws UnavailableException {
-        LOG.debug("initializeLostBookieRecoveryDelay()");
+        if (LOG.isDebugEnabled()) {
+            LOG.debug("initializeLostBookieRecoveryDelay()");
+        }
         try {
             zkc.create(lostBookieRecoveryDelayZnode, Integer.toString(lostBookieRecoveryDelay).getBytes(UTF_8),
                     Ids.OPEN_ACL_UNSAFE, CreateMode.PERSISTENT);
@@ -838,7 +838,9 @@ public class ZkLedgerUnderreplicationManager implements LedgerUnderreplicationMa
 
     @Override
     public void setLostBookieRecoveryDelay(int lostBookieRecoveryDelay) throws UnavailableException {
-        LOG.debug("setLostBookieRecoveryDelay()");
+        if (LOG.isDebugEnabled()) {
+            LOG.debug("setLostBookieRecoveryDelay()");
+        }
         try {
             if (zkc.exists(lostBookieRecoveryDelayZnode, false) != null) {
                 zkc.setData(lostBookieRecoveryDelayZnode, Integer.toString(lostBookieRecoveryDelay).getBytes(UTF_8),
@@ -858,7 +860,9 @@ public class ZkLedgerUnderreplicationManager implements LedgerUnderreplicationMa
 
     @Override
     public int getLostBookieRecoveryDelay() throws UnavailableException {
-        LOG.debug("getLostBookieRecoveryDelay()");
+        if (LOG.isDebugEnabled()) {
+            LOG.debug("getLostBookieRecoveryDelay()");
+        }
         try {
             byte[] data = zkc.getData(lostBookieRecoveryDelayZnode, false, null);
             return Integer.parseInt(new String(data, UTF_8));
@@ -871,10 +875,11 @@ public class ZkLedgerUnderreplicationManager implements LedgerUnderreplicationMa
         }
     }
 
-    @SuppressWarnings("deprecation")
     @Override
     public void notifyUnderReplicationLedgerChanged(GenericCallback<Void> cb) throws UnavailableException {
-        LOG.debug("notifyUnderReplicationLedgerChanged()");
+        if (LOG.isDebugEnabled()) {
+            LOG.debug("notifyUnderReplicationLedgerChanged()");
+        }
         Watcher w = new Watcher() {
             @Override
             public void process(WatchedEvent e) {
@@ -896,7 +901,9 @@ public class ZkLedgerUnderreplicationManager implements LedgerUnderreplicationMa
 
     @Override
     public void notifyLostBookieRecoveryDelayChanged(GenericCallback<Void> cb) throws UnavailableException {
-        LOG.debug("notifyLostBookieRecoveryDelayChanged()");
+        if (LOG.isDebugEnabled()) {
+            LOG.debug("notifyLostBookieRecoveryDelayChanged()");
+        }
         Watcher w = new Watcher() {
             @Override
             public void process(WatchedEvent e) {
