@@ -1,4 +1,4 @@
-/**
+/*
  * Licensed to the Apache Software Foundation (ASF) under one
  * or more contributor license agreements.  See the NOTICE file
  * distributed with this work for additional information
@@ -27,7 +27,6 @@ import java.util.Random;
 import java.util.Set;
 import java.util.concurrent.locks.ReadWriteLock;
 import java.util.concurrent.locks.ReentrantReadWriteLock;
-
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
@@ -422,7 +421,7 @@ public class NetworkTopologyImpl implements NetworkTopology {
             Node rack = getNodeForNetworkLocation(node);
             if (rack != null && !(rack instanceof InnerNode)) {
                 LOG.error("Unexpected data node {} at an illegal network location", node);
-                throw new IllegalArgumentException("Unexpected data node " + node.toString()
+                throw new IllegalArgumentException("Unexpected data node " + node
                         + " at an illegal network location");
             }
             if (clusterMap.add(node)) {
@@ -437,7 +436,7 @@ public class NetworkTopologyImpl implements NetworkTopology {
                 }
             }
             if (LOG.isDebugEnabled()) {
-                LOG.debug("NetworkTopology became:\n" + this.toString());
+                LOG.debug("NetworkTopology became:\n" + this);
             }
         } finally {
             netlock.writeLock().unlock();
@@ -506,12 +505,9 @@ public class NetworkTopologyImpl implements NetworkTopology {
                 if (rack == null) {
                     numOfRacks--;
                 }
-                if (clusterMap.numOfLeaves == 0) {
-                    depthOfAllLeaves = -1;
-                }
             }
             if (LOG.isDebugEnabled()) {
-                LOG.debug("NetworkTopology became:\n" + this.toString());
+                LOG.debug("NetworkTopology became:\n" + this);
             }
         } finally {
             netlock.writeLock().unlock();
