@@ -1,4 +1,4 @@
-/**
+/*
  *
  * Licensed to the Apache Software Foundation (ASF) under one
  * or more contributor license agreements.  See the NOTICE file
@@ -28,24 +28,21 @@ import com.google.common.cache.Cache;
 import com.google.common.cache.CacheBuilder;
 import com.google.common.util.concurrent.ThreadFactoryBuilder;
 import com.google.protobuf.ByteString;
-
 import io.netty.buffer.ByteBufAllocator;
 import io.netty.channel.Channel;
+import io.netty.channel.group.ChannelGroup;
 import io.netty.handler.ssl.SslHandler;
 import io.netty.util.HashedWheelTimer;
 import io.netty.util.concurrent.Future;
 import io.netty.util.concurrent.GenericFutureListener;
-
 import java.util.Optional;
 import java.util.concurrent.ExecutorService;
 import java.util.concurrent.RejectedExecutionException;
 import java.util.concurrent.Semaphore;
 import java.util.concurrent.TimeUnit;
 import java.util.function.Consumer;
-
 import lombok.AccessLevel;
 import lombok.Getter;
-
 import org.apache.bookkeeper.auth.AuthProviderFactoryFactory;
 import org.apache.bookkeeper.auth.AuthToken;
 import org.apache.bookkeeper.bookie.Bookie;
@@ -122,6 +119,8 @@ public class BookieRequestProcessor implements RequestProcessor {
     final Semaphore addsSemaphore;
     final Semaphore readsSemaphore;
 
+    final ChannelGroup allChannels;
+
     // to temporary blacklist channels
     final Optional<Cache<Channel, Boolean>> blacklistedChannels;
     final Consumer<Channel> onResponseTimeout;
@@ -131,9 +130,11 @@ public class BookieRequestProcessor implements RequestProcessor {
     private final boolean throttleReadResponses;
 
     public BookieRequestProcessor(ServerConfiguration serverCfg, Bookie bookie, StatsLogger statsLogger,
-            SecurityHandlerFactory shFactory, ByteBufAllocator allocator) throws SecurityException {
+                                  SecurityHandlerFactory shFactory, ByteBufAllocator allocator,
+                                  ChannelGroup allChannels) throws SecurityException {
         this.serverCfg = serverCfg;
         this.allocator = allocator;
+        this.allChannels = allChannels;
         this.waitTimeoutOnBackpressureMillis = serverCfg.getWaitTimeoutOnResponseBackpressureMillis();
         this.preserveMdcForTaskExecution = serverCfg.getPreserveMdcForTaskExecution();
         this.bookie = bookie;
@@ -303,7 +304,8 @@ public class BookieRequestProcessor implements RequestProcessor {
     }
 
     @Override
-    public void processRequest(Object msg, Channel c) {
+    public void processRequest(Object msg, BookieRequestHandler requestHandler) {
+        Channel channel = requestHandler.ctx().channel();
         // If we can decode this packet as a Request protobuf packet, process
         // it as a version 3 packet. Else, just use the old protocol.
         if (msg instanceof BookkeeperProtocol.Request) {
@@ -313,16 +315,16 @@ public class BookieRequestProcessor implements RequestProcessor {
                 BookkeeperProtocol.BKPacketHeader header = r.getHeader();
                 switch (header.getOperation()) {
                     case ADD_ENTRY:
-                        processAddRequestV3(r, c);
+                        processAddRequestV3(r, requestHandler);
                         break;
                     case READ_ENTRY:
-                        processReadRequestV3(r, c);
+                        processReadRequestV3(r, requestHandler);
                         break;
                     case FORCE_LEDGER:
-                        processForceLedgerRequestV3(r, c);
+                        processForceLedgerRequestV3(r, requestHandler);
                         break;
                     case AUTH:
-                        LOG.info("Ignoring auth operation from client {}", c.remoteAddress());
+                        LOG.info("Ignoring auth operation from client {}", channel.remoteAddress());
                         BookkeeperProtocol.AuthMessage message = BookkeeperProtocol.AuthMessage
                                 .newBuilder()
                                 .setAuthPluginName(AuthProviderFactoryFactory.AUTHENTICATION_DISABLED_PLUGIN_NAME)
@@ -332,29 +334,29 @@ public class BookieRequestProcessor implements RequestProcessor {
                                 .newBuilder().setHeader(r.getHeader())
                                 .setStatus(BookkeeperProtocol.StatusCode.EOK)
                                 .setAuthResponse(message);
-                        c.writeAndFlush(authResponse.build());
+                        channel.writeAndFlush(authResponse.build());
                         break;
                     case WRITE_LAC:
-                        processWriteLacRequestV3(r, c);
+                        processWriteLacRequestV3(r, requestHandler);
                         break;
                     case READ_LAC:
-                        processReadLacRequestV3(r, c);
+                        processReadLacRequestV3(r, requestHandler);
                         break;
                     case GET_BOOKIE_INFO:
-                        processGetBookieInfoRequestV3(r, c);
+                        processGetBookieInfoRequestV3(r, requestHandler);
                         break;
                     case START_TLS:
-                        processStartTLSRequestV3(r, c);
+                        processStartTLSRequestV3(r, requestHandler);
                         break;
                     case GET_LIST_OF_ENTRIES_OF_LEDGER:
-                        processGetListOfEntriesOfLedgerProcessorV3(r, c);
+                        processGetListOfEntriesOfLedgerProcessorV3(r, requestHandler);
                         break;
                     default:
                         LOG.info("Unknown operation type {}", header.getOperation());
                         BookkeeperProtocol.Response.Builder response =
                                 BookkeeperProtocol.Response.newBuilder().setHeader(r.getHeader())
                                         .setStatus(BookkeeperProtocol.StatusCode.EBADREQ);
-                        c.writeAndFlush(response.build());
+                        channel.writeAndFlush(response.build());
                         if (statsEnabled) {
                             bkStats.getOpStats(BKStats.STATS_UNKNOWN).incrementFailedOps();
                         }
@@ -369,26 +371,27 @@ public class BookieRequestProcessor implements RequestProcessor {
             switch (r.getOpCode()) {
                 case BookieProtocol.ADDENTRY:
                     checkArgument(r instanceof BookieProtocol.ParsedAddRequest);
-                    processAddRequest((BookieProtocol.ParsedAddRequest) r, c);
+                    processAddRequest((BookieProtocol.ParsedAddRequest) r, requestHandler);
                     break;
                 case BookieProtocol.READENTRY:
                     checkArgument(r instanceof BookieProtocol.ReadRequest);
-                    processReadRequest((BookieProtocol.ReadRequest) r, c);
+                    processReadRequest((BookieProtocol.ReadRequest) r, requestHandler);
                     break;
                 case BookieProtocol.AUTH:
-                    LOG.info("Ignoring auth operation from client {}", c.remoteAddress());
+                    LOG.info("Ignoring auth operation from client {}",
+                            requestHandler.ctx().channel().remoteAddress());
                     BookkeeperProtocol.AuthMessage message = BookkeeperProtocol.AuthMessage
                             .newBuilder()
                             .setAuthPluginName(AuthProviderFactoryFactory.AUTHENTICATION_DISABLED_PLUGIN_NAME)
                             .setPayload(ByteString.copyFrom(AuthToken.NULL.getData()))
                             .build();
 
-                    c.writeAndFlush(new BookieProtocol.AuthResponse(
+                    channel.writeAndFlush(new BookieProtocol.AuthResponse(
                             BookieProtocol.CURRENT_PROTOCOL_VERSION, message));
                     break;
                 default:
                     LOG.error("Unknown op type {}, sending error", r.getOpCode());
-                    c.writeAndFlush(ResponseBuilder.buildErrorResponse(BookieProtocol.EBADREQ, r));
+                    channel.writeAndFlush(ResponseBuilder.buildErrorResponse(BookieProtocol.EBADREQ, r));
                     if (statsEnabled) {
                         bkStats.getOpStats(BKStats.STATS_UNKNOWN).incrementFailedOps();
                     }
@@ -406,8 +409,9 @@ public class BookieRequestProcessor implements RequestProcessor {
         }
     }
 
-    private void processWriteLacRequestV3(final BookkeeperProtocol.Request r, final Channel c) {
-        WriteLacProcessorV3 writeLac = new WriteLacProcessorV3(r, c, this);
+    private void processWriteLacRequestV3(final BookkeeperProtocol.Request r,
+                                          final BookieRequestHandler requestHandler) {
+        WriteLacProcessorV3 writeLac = new WriteLacProcessorV3(r, requestHandler, this);
         if (null == writeThreadPool) {
             writeLac.run();
         } else {
@@ -415,8 +419,9 @@ public class BookieRequestProcessor implements RequestProcessor {
         }
     }
 
-    private void processReadLacRequestV3(final BookkeeperProtocol.Request r, final Channel c) {
-        ReadLacProcessorV3 readLac = new ReadLacProcessorV3(r, c, this);
+    private void processReadLacRequestV3(final BookkeeperProtocol.Request r,
+                                         final BookieRequestHandler requestHandler) {
+        ReadLacProcessorV3 readLac = new ReadLacProcessorV3(r, requestHandler, this);
         if (null == readThreadPool) {
             readLac.run();
         } else {
@@ -424,8 +429,8 @@ public class BookieRequestProcessor implements RequestProcessor {
         }
     }
 
-    private void processAddRequestV3(final BookkeeperProtocol.Request r, final Channel c) {
-        WriteEntryProcessorV3 write = new WriteEntryProcessorV3(r, c, this);
+    private void processAddRequestV3(final BookkeeperProtocol.Request r, final BookieRequestHandler requestHandler) {
+        WriteEntryProcessorV3 write = new WriteEntryProcessorV3(r, requestHandler, this);
 
         final OrderedExecutor threadPool;
         if (RequestUtils.isHighPriority(r)) {
@@ -459,8 +464,9 @@ public class BookieRequestProcessor implements RequestProcessor {
         }
     }
 
-    private void processForceLedgerRequestV3(final BookkeeperProtocol.Request r, final Channel c) {
-        ForceLedgerProcessorV3 forceLedger = new ForceLedgerProcessorV3(r, c, this);
+    private void processForceLedgerRequestV3(final BookkeeperProtocol.Request r,
+                                             final BookieRequestHandler requestHandler) {
+        ForceLedgerProcessorV3 forceLedger = new ForceLedgerProcessorV3(r, requestHandler, this);
 
         final OrderedExecutor threadPool;
         if (RequestUtils.isHighPriority(r)) {
@@ -496,19 +502,20 @@ public class BookieRequestProcessor implements RequestProcessor {
         }
     }
 
-    private void processReadRequestV3(final BookkeeperProtocol.Request r, final Channel c) {
-        ExecutorService fenceThread = null == highPriorityThreadPool ? null : highPriorityThreadPool.chooseThread(c);
+    private void processReadRequestV3(final BookkeeperProtocol.Request r, final BookieRequestHandler requestHandler) {
+        ExecutorService fenceThread = null == highPriorityThreadPool ? null :
+                highPriorityThreadPool.chooseThread(requestHandler.ctx());
 
         final ReadEntryProcessorV3 read;
         final OrderedExecutor threadPool;
         if (RequestUtils.isLongPollReadRequest(r.getReadRequest())) {
-            ExecutorService lpThread = longPollThreadPool.chooseThread(c);
+            ExecutorService lpThread = longPollThreadPool.chooseThread(requestHandler.ctx());
 
-            read = new LongPollReadEntryProcessorV3(r, c, this, fenceThread,
+            read = new LongPollReadEntryProcessorV3(r, requestHandler, this, fenceThread,
                                                     lpThread, requestTimer);
             threadPool = longPollThreadPool;
         } else {
-            read = new ReadEntryProcessorV3(r, c, this, fenceThread);
+            read = new ReadEntryProcessorV3(r, requestHandler, this, fenceThread);
 
             // If it's a high priority read (fencing or as part of recovery process), we want to make sure it
             // gets executed as fast as possible, so bypass the normal readThreadPool
@@ -548,13 +555,16 @@ public class BookieRequestProcessor implements RequestProcessor {
         }
     }
 
-    private void processStartTLSRequestV3(final BookkeeperProtocol.Request r, final Channel c) {
+    private void processStartTLSRequestV3(final BookkeeperProtocol.Request r,
+                                          final BookieRequestHandler requestHandler) {
         BookkeeperProtocol.Response.Builder response = BookkeeperProtocol.Response.newBuilder();
         BookkeeperProtocol.BKPacketHeader.Builder header = BookkeeperProtocol.BKPacketHeader.newBuilder();
         header.setVersion(BookkeeperProtocol.ProtocolVersion.VERSION_THREE);
         header.setOperation(r.getHeader().getOperation());
         header.setTxnId(r.getHeader().getTxnId());
         response.setHeader(header.build());
+        final Channel c = requestHandler.ctx().channel();
+
         if (shFactory == null) {
             LOG.error("Got StartTLS request but TLS not configured");
             response.setStatus(BookkeeperProtocol.StatusCode.EBADREQ);
@@ -600,8 +610,9 @@ public class BookieRequestProcessor implements RequestProcessor {
         }
     }
 
-    private void processGetBookieInfoRequestV3(final BookkeeperProtocol.Request r, final Channel c) {
-        GetBookieInfoProcessorV3 getBookieInfo = new GetBookieInfoProcessorV3(r, c, this);
+    private void processGetBookieInfoRequestV3(final BookkeeperProtocol.Request r,
+                                               final BookieRequestHandler requestHandler) {
+        GetBookieInfoProcessorV3 getBookieInfo = new GetBookieInfoProcessorV3(r, requestHandler, this);
         if (null == readThreadPool) {
             getBookieInfo.run();
         } else {
@@ -609,9 +620,10 @@ public class BookieRequestProcessor implements RequestProcessor {
         }
     }
 
-    private void processGetListOfEntriesOfLedgerProcessorV3(final BookkeeperProtocol.Request r, final Channel c) {
-        GetListOfEntriesOfLedgerProcessorV3 getListOfEntriesOfLedger = new GetListOfEntriesOfLedgerProcessorV3(r, c,
-                this);
+    private void processGetListOfEntriesOfLedgerProcessorV3(final BookkeeperProtocol.Request r,
+                                                            final BookieRequestHandler requestHandler) {
+        GetListOfEntriesOfLedgerProcessorV3 getListOfEntriesOfLedger =
+                new GetListOfEntriesOfLedgerProcessorV3(r, requestHandler, this);
         if (null == readThreadPool) {
             getListOfEntriesOfLedger.run();
         } else {
@@ -619,8 +631,8 @@ public class BookieRequestProcessor implements RequestProcessor {
         }
     }
 
-    private void processAddRequest(final BookieProtocol.ParsedAddRequest r, final Channel c) {
-        WriteEntryProcessor write = WriteEntryProcessor.create(r, c, this);
+    private void processAddRequest(final BookieProtocol.ParsedAddRequest r, final BookieRequestHandler requestHandler) {
+        WriteEntryProcessor write = WriteEntryProcessor.create(r, requestHandler, this);
 
         // If it's a high priority add (usually as part of recovery process), we want to make sure it gets
         // executed as fast as possible, so bypass the normal writeThreadPool and execute in highPriorityThreadPool
@@ -647,17 +659,15 @@ public class BookieRequestProcessor implements RequestProcessor {
                     BookieProtocol.ETOOMANYREQUESTS,
                     ResponseBuilder.buildErrorResponse(BookieProtocol.ETOOMANYREQUESTS, r),
                     requestStats.getAddRequestStats());
-                r.release();
-                r.recycle();
-                write.recycle();
             }
         }
     }
 
-    private void processReadRequest(final BookieProtocol.ReadRequest r, final Channel c) {
+    private void processReadRequest(final BookieProtocol.ReadRequest r, final BookieRequestHandler requestHandler) {
         ExecutorService fenceThreadPool =
-                null == highPriorityThreadPool ? null : highPriorityThreadPool.chooseThread(c);
-        ReadEntryProcessor read = ReadEntryProcessor.create(r, c, this, fenceThreadPool, throttleReadResponses);
+                null == highPriorityThreadPool ? null : highPriorityThreadPool.chooseThread(requestHandler.ctx());
+        ReadEntryProcessor read = ReadEntryProcessor.create(r, requestHandler,
+                this, fenceThreadPool, throttleReadResponses);
 
         // If it's a high priority read (fencing or as part of recovery process), we want to make sure it
         // gets executed as fast as possible, so bypass the normal readThreadPool
@@ -685,7 +695,6 @@ public class BookieRequestProcessor implements RequestProcessor {
                     ResponseBuilder.buildErrorResponse(BookieProtocol.ETOOMANYREQUESTS, r),
                     requestStats.getReadRequestStats());
                 onReadRequestFinish();
-                read.recycle();
             }
         }
     }
