@@ -1,4 +1,4 @@
-/**
+/*
  *
  * Licensed to the Apache Software Foundation (ASF) under one
  * or more contributor license agreements.  See the NOTICE file
@@ -27,7 +27,6 @@ import static org.junit.Assert.assertTrue;
 import com.google.common.collect.Lists;
 import java.io.File;
 import java.util.List;
-
 import org.apache.bookkeeper.bookie.BookieException;
 import org.apache.bookkeeper.bookie.EntryLogMetadata;
 import org.apache.bookkeeper.conf.ServerConfiguration;
@@ -57,7 +56,7 @@ public class PersistentEntryLogMetadataMapTest {
     @Test
     public void simple() throws Exception {
         File tmpDir = tempFolder.newFolder("metadata-cache");
-        String path = tmpDir.getAbsolutePath().toString();
+        String path = tmpDir.getAbsolutePath();
         PersistentEntryLogMetadataMap entryMetadataMap = new PersistentEntryLogMetadataMap(path, configuration);
 
         List<EntryLogMetadata> metadatas = Lists.newArrayList();
@@ -118,7 +117,7 @@ public class PersistentEntryLogMetadataMapTest {
     @Test
     public void closeAndOpen() throws Exception {
         File tmpDir = tempFolder.newFolder();
-        String path = tmpDir.getAbsolutePath().toString();
+        String path = tmpDir.getAbsolutePath();
         PersistentEntryLogMetadataMap entryMetadataMap = new PersistentEntryLogMetadataMap(path, configuration);
 
         List<EntryLogMetadata> metadatas = Lists.newArrayList();
