@@ -1,4 +1,4 @@
-/**
+/*
  *
  * Licensed to the Apache Software Foundation (ASF) under one
  * or more contributor license agreements.  See the NOTICE file
@@ -26,9 +26,9 @@ import static org.apache.bookkeeper.client.BookKeeperClientStats.CLIENT_SCOPE;
 import static org.apache.bookkeeper.client.BookKeeperClientStats.READ_OP_DM;
 import static org.junit.Assert.assertEquals;
 import static org.junit.Assert.assertFalse;
-import static org.junit.Assert.assertNotEquals;
 import static org.junit.Assert.assertTrue;
 import static org.junit.Assert.fail;
+
 import com.google.common.collect.Lists;
 import io.netty.buffer.AbstractByteBufAllocator;
 import io.netty.buffer.ByteBuf;
@@ -36,11 +36,10 @@ import io.netty.buffer.PooledByteBufAllocator;
 import io.netty.buffer.Unpooled;
 import io.netty.buffer.UnpooledByteBufAllocator;
 import java.io.IOException;
-import java.net.URI;
 import java.nio.ByteBuffer;
 import java.util.ArrayList;
 import java.util.Arrays;
-import java.util.Collections;
+import java.util.Collection;
 import java.util.Enumeration;
 import java.util.HashMap;
 import java.util.List;
@@ -61,25 +60,17 @@ import org.apache.bookkeeper.client.api.LedgerEntries;
 import org.apache.bookkeeper.client.api.ReadHandle;
 import org.apache.bookkeeper.client.api.WriteAdvHandle;
 import org.apache.bookkeeper.conf.ServerConfiguration;
-import org.apache.bookkeeper.meta.LedgerManagerFactory;
 import org.apache.bookkeeper.meta.LedgerMetadataSerDe;
-import org.apache.bookkeeper.meta.LedgerUnderreplicationManager;
 import org.apache.bookkeeper.meta.LongHierarchicalLedgerManagerFactory;
-import org.apache.bookkeeper.meta.MetadataBookieDriver;
-import org.apache.bookkeeper.meta.MetadataDrivers;
-import org.apache.bookkeeper.meta.zk.ZKMetadataDriverBase;
 import org.apache.bookkeeper.net.BookieId;
-import org.apache.bookkeeper.replication.ReplicationTestUtil;
-import org.apache.bookkeeper.replication.ReplicationWorker;
-import org.apache.bookkeeper.stats.NullStatsLogger;
 import org.apache.bookkeeper.test.BookKeeperClusterTestCase;
 import org.apache.bookkeeper.test.TestStatsProvider;
-import org.apache.bookkeeper.util.BookKeeperConstants;
 import org.apache.commons.lang3.tuple.Pair;
-import org.awaitility.Awaitility;
 import org.junit.Assert;
 import org.junit.Before;
 import org.junit.Test;
+import org.junit.runner.RunWith;
+import org.junit.runners.Parameterized;
 import org.powermock.reflect.Whitebox;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
@@ -87,12 +78,26 @@ import org.slf4j.LoggerFactory;
 /**
  * Testing ledger write entry cases.
  */
+@RunWith(Parameterized.class)
 public class BookieWriteLedgerTest extends
     BookKeeperClusterTestCase implements AddCallback {
 
     private static final Logger LOG = LoggerFactory
             .getLogger(BookieWriteLedgerTest.class);
 
+    @Parameterized.Parameters
+    public static Collection<Object[]> data() {
+        return Arrays.asList(new Object[][] {
+            { true, true }, { true, false }, { false, true }, { false, false }
+        });
+    }
+
+    @Parameterized.Parameter(0)
+    public boolean useV2;
+
+    @Parameterized.Parameter(1)
+    public boolean writeJournal;
+
     byte[] ledgerPassword = "aaa".getBytes();
     LedgerHandle lh, lh2;
     Enumeration<LedgerEntry> ls;
@@ -131,12 +136,14 @@ public class BookieWriteLedgerTest extends
         String ledgerManagerFactory = "org.apache.bookkeeper.meta.HierarchicalLedgerManagerFactory";
         // set ledger manager
         baseConf.setLedgerManagerFactoryClassName(ledgerManagerFactory);
+        baseConf.setJournalWriteData(writeJournal);
         /*
          * 'testLedgerCreateAdvWithLedgerIdInLoop2' testcase relies on skipListSizeLimit,
          * so setting it to some small value for making that testcase lite.
          */
         baseConf.setSkipListSizeLimit(4 * 1024 * 1024);
         baseClientConf.setLedgerManagerFactoryClassName(ledgerManagerFactory);
+        baseClientConf.setUseV2WireProtocol(useV2);
     }
 
     /**
@@ -827,7 +834,9 @@ public class BookieWriteLedgerTest extends
                 ledgerId %= 9999999999L;
             }
 
-            LOG.debug("Iteration: {}  LedgerId: {}", lc, ledgerId);
+            if (LOG.isDebugEnabled()) {
+                LOG.debug("Iteration: {}  LedgerId: {}", lc, ledgerId);
+            }
             lh = bkc.createLedgerAdv(ledgerId, 5, 3, 2, digestType, ledgerPassword, null);
             lhArray[lc] = lh;
 
@@ -854,7 +863,9 @@ public class BookieWriteLedgerTest extends
         for (int lc = 0; lc < ledgerCount; lc++) {
             // Read and verify
             long lid = lhArray[lc].getId();
-            LOG.debug("readEntries for lc: {} ledgerId: {} ", lc, lhArray[lc].getId());
+            if (LOG.isDebugEnabled()) {
+                LOG.debug("readEntries for lc: {} ledgerId: {} ", lc, lhArray[lc].getId());
+            }
             readEntriesAndValidateDataArray(lhArray[lc], entryList.get(lc));
             lhArray[lc].close();
             bkc.deleteLedger(lid);
@@ -913,7 +924,9 @@ public class BookieWriteLedgerTest extends
         // wait for all entries to be acknowledged for the first ledger
         synchronized (syncObj1) {
             while (syncObj1.counter < 1) {
-                LOG.debug("Entries counter = " + syncObj1.counter);
+                if (LOG.isDebugEnabled()) {
+                    LOG.debug("Entries counter = " + syncObj1.counter);
+                }
                 syncObj1.wait();
             }
             assertEquals(BKException.Code.OK, syncObj1.rc);
@@ -921,7 +934,9 @@ public class BookieWriteLedgerTest extends
         // wait for all entries to be acknowledged for the second ledger
         synchronized (syncObj2) {
             while (syncObj2.counter < 1) {
-                LOG.debug("Entries counter = " + syncObj2.counter);
+                if (LOG.isDebugEnabled()) {
+                    LOG.debug("Entries counter = " + syncObj2.counter);
+                }
                 syncObj2.wait();
             }
             assertEquals(BKException.Code.OK, syncObj2.rc);
@@ -1438,68 +1453,6 @@ public class BookieWriteLedgerTest extends
         bkc.deleteLedger(lh.ledgerId);
     }
 
-    @Test
-    public void testReadLacNotSameWithMetadataLedgerReplication() throws Exception {
-       lh = bkc.createLedger(3, 3, 2, digestType, ledgerPassword);
-        for (int i = 0; i < 10; ++i) {
-            ByteBuffer entry = ByteBuffer.allocate(4);
-            entry.putInt(rng.nextInt(maxInt));
-            entry.position(0);
-            lh.addEntry(entry.array());
-        }
-
-        List<BookieId> ensemble = lh.getLedgerMetadata().getAllEnsembles().entrySet().iterator().next().getValue();
-        assertEquals(1, lh.getLedgerMetadata().getAllEnsembles().size());
-        killBookie(ensemble.get(1));
-
-        try {
-            lh.ensembleChangeLoop(ensemble, Collections.singletonMap(1, ensemble.get(1)));
-        } catch (Exception e) {
-            fail();
-        }
-
-        LedgerHandle lh1 = bkc.openLedgerNoRecovery(lh.ledgerId, digestType, ledgerPassword);
-        assertEquals(2, lh1.getLedgerMetadata().getAllEnsembles().size());
-        List<BookieId> firstEnsemble = lh1.getLedgerMetadata().getAllEnsembles().firstEntry().getValue();
-
-        long entryId = lh1.getLedgerMetadata().getAllEnsembles().lastEntry().getKey() - 1;
-        try {
-            lh1.readAsync(entryId, entryId).get();
-            fail();
-        } catch (Exception e) {
-            LOG.info("Failed to read entry: {} ", entryId, e);
-        }
-
-        MetadataBookieDriver driver = MetadataDrivers.getBookieDriver(
-            URI.create(baseConf.getMetadataServiceUri()));
-        driver.initialize(
-            baseConf,
-            NullStatsLogger.INSTANCE);
-        // initialize urReplicationManager
-        LedgerManagerFactory mFactory = driver.getLedgerManagerFactory();
-        LedgerUnderreplicationManager underReplicationManager = mFactory.newLedgerUnderreplicationManager();
-        baseConf.setOpenLedgerRereplicationGracePeriod(String.valueOf(30));
-
-
-        ReplicationWorker replicationWorker = new ReplicationWorker(baseConf);
-        replicationWorker.start();
-        String basePath = ZKMetadataDriverBase.resolveZkLedgersRootPath(baseClientConf) + '/'
-            + BookKeeperConstants.UNDER_REPLICATION_NODE
-            + BookKeeperConstants.DEFAULT_ZK_LEDGERS_ROOT_PATH;
-
-        try {
-            underReplicationManager.markLedgerUnderreplicated(lh1.getId(), ensemble.get(1).toString());
-
-            Awaitility.waitAtMost(30, TimeUnit.SECONDS).untilAsserted(() ->
-                assertFalse(ReplicationTestUtil.isLedgerInUnderReplication(zkc, lh1.getId(), basePath))
-            );
-
-            assertNotEquals(firstEnsemble, lh1.getLedgerMetadata().getAllEnsembles().firstEntry().getValue());
-        } finally {
-            replicationWorker.shutdown();
-        }
-    }
-
     @Test
     public void testLedgerMetadataTest() throws Exception {
         baseClientConf.setLedgerMetadataFormatVersion(LedgerMetadataSerDe.METADATA_FORMAT_VERSION_2);
@@ -1517,10 +1470,12 @@ public class BookieWriteLedgerTest extends
             ByteBuffer origbb = ByteBuffer.wrap(entries.get(index++));
             Integer origEntry = origbb.getInt();
             ByteBuffer result = ByteBuffer.wrap(ls.nextElement().getEntry());
-            LOG.debug("Length of result: " + result.capacity());
-            LOG.debug("Original entry: " + origEntry);
             Integer retrEntry = result.getInt();
-            LOG.debug("Retrieved entry: " + retrEntry);
+            if (LOG.isDebugEnabled()) {
+                LOG.debug("Length of result: " + result.capacity());
+                LOG.debug("Original entry: " + origEntry);
+                LOG.debug("Retrieved entry: " + retrEntry);
+            }
             assertTrue("Checking entry " + index + " for equality", origEntry
                     .equals(retrEntry));
         }
@@ -1547,8 +1502,10 @@ public class BookieWriteLedgerTest extends
         while (ls.hasMoreElements()) {
             byte[] originalData = entries.get(index++);
             byte[] receivedData = ls.nextElement().getEntry();
-            LOG.debug("Length of originalData: {}", originalData.length);
-            LOG.debug("Length of receivedData: {}", receivedData.length);
+            if (LOG.isDebugEnabled()) {
+                LOG.debug("Length of originalData: {}", originalData.length);
+                LOG.debug("Length of receivedData: {}", receivedData.length);
+            }
             assertEquals(
                     String.format("LedgerID: %d EntryID: %d OriginalDataLength: %d ReceivedDataLength: %d", lh.getId(),
                             (index - 1), originalData.length, receivedData.length),
