@@ -1,4 +1,4 @@
-/**
+/*
  *
  * Licensed to the Apache Software Foundation (ASF) under one
  * or more contributor license agreements.  See the NOTICE file
@@ -152,14 +152,18 @@ public class AuditorLedgerCheckerTest extends BookKeeperClusterTestCase {
             AuditorElector auditorElector = new AuditorElector(addr, baseConf);
             auditorElectors.put(addr, auditorElector);
             auditorElector.start();
-            LOG.debug("Starting Auditor Elector");
+            if (LOG.isDebugEnabled()) {
+                LOG.debug("Starting Auditor Elector");
+            }
         }
     }
 
     private void stopAuditorElectors() throws Exception {
         for (AuditorElector auditorElector : auditorElectors.values()) {
             auditorElector.shutdown();
-            LOG.debug("Stopping Auditor Elector!");
+            if (LOG.isDebugEnabled()) {
+                LOG.debug("Stopping Auditor Elector!");
+            }
         }
     }
 
@@ -170,7 +174,9 @@ public class AuditorLedgerCheckerTest extends BookKeeperClusterTestCase {
     public void testSimpleLedger() throws Exception {
         LedgerHandle lh1 = createAndAddEntriesToLedger();
         Long ledgerId = lh1.getId();
-        LOG.debug("Created ledger : " + ledgerId);
+        if (LOG.isDebugEnabled()) {
+            LOG.debug("Created ledger : " + ledgerId);
+        }
         ledgerList.add(ledgerId);
         lh1.close();
 
@@ -181,7 +187,9 @@ public class AuditorLedgerCheckerTest extends BookKeeperClusterTestCase {
         String shutdownBookie = shutdownBookie(bkShutdownIndex);
 
         // grace period for publishing the bk-ledger
-        LOG.debug("Waiting for ledgers to be marked as under replicated");
+        if (LOG.isDebugEnabled()) {
+            LOG.debug("Waiting for ledgers to be marked as under replicated");
+        }
         waitForAuditToComplete();
         underReplicaLatch.await(5, TimeUnit.SECONDS);
         Map<Long, String> urLedgerData = getUrLedgerData(urLedgerList);
@@ -210,7 +218,9 @@ public class AuditorLedgerCheckerTest extends BookKeeperClusterTestCase {
         LedgerHandle lh1 = createAndAddEntriesToLedger();
         LedgerHandle lh2 = createAndAddEntriesToLedger();
 
-        LOG.debug("Created following ledgers : {}, {}", lh1, lh2);
+        if (LOG.isDebugEnabled()) {
+            LOG.debug("Created following ledgers : {}, {}", lh1, lh2);
+        }
 
         int bkShutdownIndex = lastBookieIndex();
         ServerConfiguration bookieConf1 = confByIndex(bkShutdownIndex);
@@ -241,7 +251,9 @@ public class AuditorLedgerCheckerTest extends BookKeeperClusterTestCase {
         String shutdownBookie = shutdownBookie(lastBookieIndex());
 
         // grace period for publishing the bk-ledger
-        LOG.debug("Waiting for ledgers to be marked as under replicated");
+        if (LOG.isDebugEnabled()) {
+            LOG.debug("Waiting for ledgers to be marked as under replicated");
+        }
         assertTrue("Ledger should be missing second replica",
                    waitForLedgerMissingReplicas(lh1.getId(), 10, shutdownBookie));
     }
@@ -250,7 +262,9 @@ public class AuditorLedgerCheckerTest extends BookKeeperClusterTestCase {
     public void testToggleLedgerReplication() throws Exception {
         LedgerHandle lh1 = createAndAddEntriesToLedger();
         ledgerList.add(lh1.getId());
-        LOG.debug("Created following ledgers : " + ledgerList);
+        if (LOG.isDebugEnabled()) {
+            LOG.debug("Created following ledgers : " + ledgerList);
+        }
 
         // failing another bookie
         CountDownLatch urReplicaLatch = registerUrLedgerWatcher(ledgerList
@@ -299,7 +313,9 @@ public class AuditorLedgerCheckerTest extends BookKeeperClusterTestCase {
     public void testReadOnlyBookieExclusionFromURLedgersCheck() throws Exception {
         LedgerHandle lh = createAndAddEntriesToLedger();
         ledgerList.add(lh.getId());
-        LOG.debug("Created following ledgers : " + ledgerList);
+        if (LOG.isDebugEnabled()) {
+            LOG.debug("Created following ledgers : " + ledgerList);
+        }
 
         int count = ledgerList.size();
         final CountDownLatch underReplicaLatch = registerUrLedgerWatcher(count);
@@ -314,7 +330,9 @@ public class AuditorLedgerCheckerTest extends BookKeeperClusterTestCase {
             .get(30, TimeUnit.SECONDS);
 
         // grace period for publishing the bk-ledger
-        LOG.debug("Waiting for Auditor to finish ledger check.");
+        if (LOG.isDebugEnabled()) {
+            LOG.debug("Waiting for Auditor to finish ledger check.");
+        }
         waitForAuditToComplete();
         assertFalse("latch should not have completed", underReplicaLatch.await(5, TimeUnit.SECONDS));
     }
@@ -327,13 +345,17 @@ public class AuditorLedgerCheckerTest extends BookKeeperClusterTestCase {
         LedgerHandle lh = createAndAddEntriesToLedger();
         long ledgerId = lh.getId();
         ledgerList.add(ledgerId);
-        LOG.debug("Created following ledgers : " + ledgerList);
+        if (LOG.isDebugEnabled()) {
+            LOG.debug("Created following ledgers : " + ledgerList);
+        }
 
         int count = ledgerList.size();
         final CountDownLatch underReplicaLatch = registerUrLedgerWatcher(count);
 
         int bkIndex = lastBookieIndex();
-        LOG.debug("Moving bookie {} {} to read only...", bkIndex, serverByIndex(bkIndex));
+        if (LOG.isDebugEnabled()) {
+            LOG.debug("Moving bookie {} {} to read only...", bkIndex, serverByIndex(bkIndex));
+        }
         ServerConfiguration bookieConf = confByIndex(bkIndex);
         BookieServer bk = serverByIndex(bkIndex);
         bookieConf.setReadOnlyModeEnabled(true);
@@ -343,14 +365,18 @@ public class AuditorLedgerCheckerTest extends BookKeeperClusterTestCase {
             .get(30, TimeUnit.SECONDS);
 
         // grace period for publishing the bk-ledger
-        LOG.debug("Waiting for Auditor to finish ledger check.");
+        if (LOG.isDebugEnabled()) {
+            LOG.debug("Waiting for Auditor to finish ledger check.");
+        }
         waitForAuditToComplete();
         assertFalse("latch should not have completed", underReplicaLatch.await(1, TimeUnit.SECONDS));
 
         String shutdownBookie = shutdownBookie(bkIndex);
 
         // grace period for publishing the bk-ledger
-        LOG.debug("Waiting for ledgers to be marked as under replicated");
+        if (LOG.isDebugEnabled()) {
+            LOG.debug("Waiting for ledgers to be marked as under replicated");
+        }
         waitForAuditToComplete();
         underReplicaLatch.await(5, TimeUnit.SECONDS);
         Map<Long, String> urLedgerData = getUrLedgerData(urLedgerList);
@@ -370,7 +396,9 @@ public class AuditorLedgerCheckerTest extends BookKeeperClusterTestCase {
     public void testInnerDelayedAuditOfLostBookies() throws Exception {
         LedgerHandle lh1 = createAndAddEntriesToLedger();
         Long ledgerId = lh1.getId();
-        LOG.debug("Created ledger : " + ledgerId);
+        if (LOG.isDebugEnabled()) {
+            LOG.debug("Created ledger : " + ledgerId);
+        }
         ledgerList.add(ledgerId);
         lh1.close();
 
@@ -383,7 +411,9 @@ public class AuditorLedgerCheckerTest extends BookKeeperClusterTestCase {
         // shutdown a non auditor bookie; choosing non-auditor to avoid another election
         String shutdownBookie = shutDownNonAuditorBookie();
 
-        LOG.debug("Waiting for ledgers to be marked as under replicated");
+        if (LOG.isDebugEnabled()) {
+            LOG.debug("Waiting for ledgers to be marked as under replicated");
+        }
         assertFalse("audit of lost bookie isn't delayed", underReplicaLatch.await(4, TimeUnit.SECONDS));
         assertEquals("under replicated ledgers identified when it was not expected", 0,
                 urLedgerList.size());
@@ -443,7 +473,9 @@ public class AuditorLedgerCheckerTest extends BookKeeperClusterTestCase {
 
         LedgerHandle lh1 = createAndAddEntriesToLedger();
         Long ledgerId = lh1.getId();
-        LOG.debug("Created ledger : " + ledgerId);
+        if (LOG.isDebugEnabled()) {
+            LOG.debug("Created ledger : " + ledgerId);
+        }
         ledgerList.add(ledgerId);
         lh1.close();
 
@@ -456,7 +488,9 @@ public class AuditorLedgerCheckerTest extends BookKeeperClusterTestCase {
         // shutdown a non auditor bookie; choosing non-auditor to avoid another election
         String shutdownBookie = shutDownNonAuditorBookie();
 
-        LOG.debug("Waiting for ledgers to be marked as under replicated");
+        if (LOG.isDebugEnabled()) {
+            LOG.debug("Waiting for ledgers to be marked as under replicated");
+        }
         assertFalse("audit of lost bookie isn't delayed", underReplicaLatch.await(4, TimeUnit.SECONDS));
         assertEquals("under replicated ledgers identified when it was not expected", 0,
                 urLedgerList.size());
@@ -483,7 +517,9 @@ public class AuditorLedgerCheckerTest extends BookKeeperClusterTestCase {
 
         LedgerHandle lh1 = createAndAddEntriesToLedger();
         Long ledgerId = lh1.getId();
-        LOG.debug("Created ledger : " + ledgerId);
+        if (LOG.isDebugEnabled()) {
+            LOG.debug("Created ledger : " + ledgerId);
+        }
         ledgerList.add(ledgerId);
         lh1.close();
 
@@ -496,7 +532,9 @@ public class AuditorLedgerCheckerTest extends BookKeeperClusterTestCase {
         // shutdown a non auditor bookie; choosing non-auditor to avoid another election
         String shutdownBookie = shutDownNonAuditorBookie();
 
-        LOG.debug("Waiting for ledgers to be marked as under replicated");
+        if (LOG.isDebugEnabled()) {
+            LOG.debug("Waiting for ledgers to be marked as under replicated");
+        }
         assertFalse("audit of lost bookie isn't delayed", underReplicaLatch.await(2, TimeUnit.SECONDS));
         assertEquals("under replicated ledgers identified when it was not expected", 0,
                 urLedgerList.size());
@@ -505,7 +543,9 @@ public class AuditorLedgerCheckerTest extends BookKeeperClusterTestCase {
         urLedgerMgr.setLostBookieRecoveryDelay(4);
 
         // since we changed the BookieRecoveryDelay period to 4, the audittask shouldn't have been executed
-        LOG.debug("Waiting for ledgers to be marked as under replicated");
+        if (LOG.isDebugEnabled()) {
+            LOG.debug("Waiting for ledgers to be marked as under replicated");
+        }
         assertFalse("audit of lost bookie isn't delayed", underReplicaLatch.await(2, TimeUnit.SECONDS));
         assertEquals("under replicated ledgers identified when it was not expected", 0,
                 urLedgerList.size());
@@ -593,7 +633,9 @@ public class AuditorLedgerCheckerTest extends BookKeeperClusterTestCase {
         Auditor auditorBookiesAuditor = getAuditorBookiesAuditor();
         LedgerHandle lh1 = createAndAddEntriesToLedger();
         Long ledgerId = lh1.getId();
-        LOG.debug("Created ledger : " + ledgerId);
+        if (LOG.isDebugEnabled()) {
+            LOG.debug("Created ledger : " + ledgerId);
+        }
         ledgerList.add(ledgerId);
         lh1.close();
 
@@ -607,7 +649,9 @@ public class AuditorLedgerCheckerTest extends BookKeeperClusterTestCase {
         // shutdown a non auditor bookie; choosing non-auditor to avoid another election
         String shutdownBookie = shutDownNonAuditorBookie();
 
-        LOG.debug("Waiting for ledgers to be marked as under replicated");
+        if (LOG.isDebugEnabled()) {
+            LOG.debug("Waiting for ledgers to be marked as under replicated");
+        }
         assertFalse("audit of lost bookie isn't delayed", underReplicaLatch.await(2, TimeUnit.SECONDS));
         assertEquals("under replicated ledgers identified when it was not expected", 0,
                 urLedgerList.size());
@@ -640,7 +684,9 @@ public class AuditorLedgerCheckerTest extends BookKeeperClusterTestCase {
         Auditor auditorBookiesAuditor = getAuditorBookiesAuditor();
         LedgerHandle lh1 = createAndAddEntriesToLedger();
         Long ledgerId = lh1.getId();
-        LOG.debug("Created ledger : " + ledgerId);
+        if (LOG.isDebugEnabled()) {
+            LOG.debug("Created ledger : " + ledgerId);
+        }
         ledgerList.add(ledgerId);
         lh1.close();
 
@@ -654,7 +700,9 @@ public class AuditorLedgerCheckerTest extends BookKeeperClusterTestCase {
         // shutdown a non auditor bookie; choosing non-auditor to avoid another election
         String shutdownBookie = shutDownNonAuditorBookie();
 
-        LOG.debug("Waiting for ledgers to be marked as under replicated");
+        if (LOG.isDebugEnabled()) {
+            LOG.debug("Waiting for ledgers to be marked as under replicated");
+        }
         assertFalse("audit of lost bookie isn't delayed", underReplicaLatch.await(2, TimeUnit.SECONDS));
         assertEquals("under replicated ledgers identified when it was not expected", 0,
                 urLedgerList.size());
@@ -691,7 +739,9 @@ public class AuditorLedgerCheckerTest extends BookKeeperClusterTestCase {
         // create a ledger with a bunch of entries
         LedgerHandle lh1 = createAndAddEntriesToLedger();
         Long ledgerId = lh1.getId();
-        LOG.debug("Created ledger : " + ledgerId);
+        if (LOG.isDebugEnabled()) {
+            LOG.debug("Created ledger : " + ledgerId);
+        }
         ledgerList.add(ledgerId);
         lh1.close();
 
@@ -744,7 +794,9 @@ public class AuditorLedgerCheckerTest extends BookKeeperClusterTestCase {
         // create a ledger with a bunch of entries
         LedgerHandle lh1 = createAndAddEntriesToLedger();
         Long ledgerId = lh1.getId();
-        LOG.debug("Created ledger : " + ledgerId);
+        if (LOG.isDebugEnabled()) {
+            LOG.debug("Created ledger : " + ledgerId);
+        }
         ledgerList.add(ledgerId);
         lh1.close();
 
@@ -857,7 +909,9 @@ public class AuditorLedgerCheckerTest extends BookKeeperClusterTestCase {
     private String shutdownBookie(int bkShutdownIndex) throws Exception {
         BookieServer bkServer = serverByIndex(bkShutdownIndex);
         String bookieAddr = bkServer.getBookieId().toString();
-        LOG.debug("Shutting down bookie:" + bookieAddr);
+        if (LOG.isDebugEnabled()) {
+            LOG.debug("Shutting down bookie:" + bookieAddr);
+        }
         killBookie(bkShutdownIndex);
         auditorElectors.get(bookieAddr).shutdown();
         auditorElectors.remove(bookieAddr);
@@ -925,7 +979,9 @@ public class AuditorLedgerCheckerTest extends BookKeeperClusterTestCase {
                     urLedgerList.add(ledgerId);
                 }
             }
-            LOG.debug("Count down and waiting for next notification");
+            if (LOG.isDebugEnabled()) {
+                LOG.debug("Count down and waiting for next notification");
+            }
             // count down and waiting for next notification
             underReplicaLatch.countDown();
         }
