@@ -23,20 +23,22 @@ import static org.junit.Assert.assertEquals;
 import static org.junit.Assert.assertFalse;
 import static org.junit.Assert.assertNotNull;
 import static org.junit.Assert.assertTrue;
+import static org.mockito.Mockito.when;
+import static org.powermock.api.mockito.PowerMockito.spy;
 
 import com.google.common.collect.Lists;
 import com.google.common.collect.Maps;
 import com.google.common.util.concurrent.UncheckedExecutionException;
-
 import java.io.File;
+import java.lang.reflect.Field;
 import java.util.HashMap;
 import java.util.LinkedHashMap;
+import java.util.List;
 import java.util.Map;
 import java.util.concurrent.Future;
-
 import lombok.Cleanup;
-
 import org.apache.bookkeeper.bookie.BookieResources;
+import org.apache.bookkeeper.bookie.LedgerStorage;
 import org.apache.bookkeeper.client.BookKeeper;
 import org.apache.bookkeeper.client.ClientUtil;
 import org.apache.bookkeeper.client.LedgerHandle;
@@ -53,8 +55,11 @@ import org.apache.bookkeeper.meta.LedgerManagerFactory;
 import org.apache.bookkeeper.meta.LedgerUnderreplicationManager;
 import org.apache.bookkeeper.meta.MetadataBookieDriver;
 import org.apache.bookkeeper.net.BookieSocketAddress;
+import org.apache.bookkeeper.proto.BookieServer;
 import org.apache.bookkeeper.replication.AuditorElector;
 import org.apache.bookkeeper.server.http.service.BookieInfoService;
+import org.apache.bookkeeper.server.http.service.BookieSanityService;
+import org.apache.bookkeeper.server.http.service.BookieSanityService.BookieSanity;
 import org.apache.bookkeeper.server.http.service.BookieStateReadOnlyService.ReadOnlyState;
 import org.apache.bookkeeper.server.http.service.BookieStateService.BookieState;
 import org.apache.bookkeeper.server.http.service.ClusterInfoService;
@@ -857,6 +862,27 @@ public class TestHttpService extends BookKeeperClusterTestCase {
         assertEquals(false, bs.isShuttingDown());
     }
 
+    @Test
+    public void testGetBookieSanity() throws Exception {
+        HttpEndpointService bookieStateServer = bkHttpServiceProvider
+                .provideHttpEndpointService(HttpServer.ApiType.BOOKIE_SANITY);
+
+        HttpServiceRequest request1 = new HttpServiceRequest(null, HttpServer.Method.GET, null);
+        ServerConfiguration conf = servers.get(0).getConfiguration();
+        BookieSanityService service = new BookieSanityService(conf);
+        HttpServiceResponse response1 = service.handle(request1);
+        assertEquals(HttpServer.StatusCode.OK.getValue(), response1.getStatusCode());
+        // run multiple iteration to validate any server side throttling doesn't
+        // fail sequential requests.
+        for (int i = 0; i < 3; i++) {
+            BookieSanity bs = JsonUtil.fromJson(response1.getBody(), BookieSanity.class);
+            assertEquals(true, bs.isPassed());
+            assertEquals(false, bs.isReadOnly());
+        }
+        HttpServiceResponse response2 = bookieStateServer.handle(request1);
+        assertEquals(HttpServer.StatusCode.OK.getValue(), response2.getStatusCode());
+    }
+
     @Test
     public void testGetBookieIsReady() throws Exception {
         HttpEndpointService bookieStateServer = bkHttpServiceProvider
@@ -995,13 +1021,151 @@ public class TestHttpService extends BookKeeperClusterTestCase {
         response = bookieReadOnlyService.handle(request);
         readOnlyState = JsonUtil.fromJson(response.getBody(), ReadOnlyState.class);
         assertFalse(readOnlyState.isReadOnly());
+    }
 
-        //forceReadonly to writable
-        baseConf.setForceReadOnlyBookie(true);
-        baseConf.setReadOnlyModeEnabled(true);
-        restartBookies();
-        request = new HttpServiceRequest(JsonUtil.toJson(new ReadOnlyState(false)), HttpServer.Method.PUT,  null);
-        response = bookieReadOnlyService.handle(request);
-        assertEquals(400, response.getStatusCode());
+    @Test
+    public void testSuspendCompaction() throws Exception {
+        HttpEndpointService suspendCompactionService = bkHttpServiceProvider
+                .provideHttpEndpointService(HttpServer.ApiType.SUSPEND_GC_COMPACTION);
+
+        HttpEndpointService resumeCompactionService = bkHttpServiceProvider
+                .provideHttpEndpointService(HttpServer.ApiType.RESUME_GC_COMPACTION);
+
+        //1,  PUT with null body, should return error
+        HttpServiceRequest request1 = new HttpServiceRequest(null, HttpServer.Method.PUT, null);
+        HttpServiceResponse response1 = suspendCompactionService.handle(request1);
+        assertEquals(HttpServer.StatusCode.BAD_REQUEST.getValue(), response1.getStatusCode());
+
+        //2,  PUT with null, should return error, because should contains "suspendMajor" or "suspendMinor"
+        String putBody2 = "{}";
+        HttpServiceRequest request2 = new HttpServiceRequest(putBody2, HttpServer.Method.PUT, null);
+        HttpServiceResponse response2 = suspendCompactionService.handle(request2);
+        assertEquals(HttpServer.StatusCode.BAD_REQUEST.getValue(), response2.getStatusCode());
+
+
+        //3,  GET before suspend, should success
+        HttpServiceRequest request3 = new HttpServiceRequest(null, HttpServer.Method.GET, null);
+        HttpServiceResponse response3 = suspendCompactionService.handle(request3);
+        assertEquals(HttpServer.StatusCode.OK.getValue(), response3.getStatusCode());
+
+        Map responseMap = JsonUtil.fromJson(
+                response3.getBody(),
+                Map.class
+        );
+        assertEquals(responseMap.get("isMajorGcSuspended"), "false");
+        assertEquals(responseMap.get("isMinorGcSuspended"), "false");
+
+
+        //2, PUT, with body, should success
+        String putBody4 = "{\"suspendMajor\": true, \"suspendMinor\": true}";
+        HttpServiceRequest request4 = new HttpServiceRequest(putBody4, HttpServer.Method.PUT, null);
+        HttpServiceResponse response4 = suspendCompactionService.handle(request4);
+        assertEquals(HttpServer.StatusCode.OK.getValue(), response4.getStatusCode());
+
+        //3,  GET after suspend, should success
+        HttpServiceRequest request5 = new HttpServiceRequest(null, HttpServer.Method.GET, null);
+        HttpServiceResponse response5 = suspendCompactionService.handle(request5);
+        assertEquals(HttpServer.StatusCode.OK.getValue(), response5.getStatusCode());
+
+        Map responseMap5 = JsonUtil.fromJson(
+                response5.getBody(),
+                Map.class
+        );
+        assertEquals(responseMap5.get("isMajorGcSuspended"), "true");
+        assertEquals(responseMap5.get("isMinorGcSuspended"), "true");
+
+
+        //2, PUT, with body, should success
+        String putBody6 = "{\"resumeMajor\": true, \"resumeMinor\": true}";
+        HttpServiceRequest request6 = new HttpServiceRequest(putBody6, HttpServer.Method.PUT, null);
+        HttpServiceResponse response6 = resumeCompactionService.handle(request6);
+        assertEquals(HttpServer.StatusCode.OK.getValue(), response6.getStatusCode());
+
+        //3,  GET after suspend, should success
+        HttpServiceRequest request7 = new HttpServiceRequest(null, HttpServer.Method.GET, null);
+        HttpServiceResponse response7 = suspendCompactionService.handle(request7);
+        assertEquals(HttpServer.StatusCode.OK.getValue(), response7.getStatusCode());
+
+        Map responseMap7 = JsonUtil.fromJson(
+                response7.getBody(),
+                Map.class
+        );
+        assertEquals(responseMap7.get("isMajorGcSuspended"), "false");
+        assertEquals(responseMap7.get("isMinorGcSuspended"), "false");
+    }
+
+    @Test
+    public void testTriggerEntryLocationCompactService() throws Exception {
+        BookieServer bookieServer = serverByIndex(numberOfBookies - 1);
+        LedgerStorage spyLedgerStorage = spy(bookieServer.getBookie().getLedgerStorage());
+        List<String> dbLocationPath = Lists.newArrayList("/data1/bookkeeper/ledgers/current/locations",
+                "/data2/bookkeeper/ledgers/current/locations");
+        when(spyLedgerStorage.getEntryLocationDBPath())
+                .thenReturn(dbLocationPath);
+
+        HashMap<String, Boolean> statusMap = Maps.newHashMap();
+        statusMap.put("/data1/bookkeeper/ledgers/current/locations", false);
+        statusMap.put("/data2/bookkeeper/ledgers/current/locations", true);
+        when(spyLedgerStorage.isEntryLocationCompacting(dbLocationPath))
+                .thenReturn(statusMap);
+
+        Field ledgerStorageField = bookieServer.getBookie().getClass().getDeclaredField("ledgerStorage");
+        ledgerStorageField.setAccessible(true);
+        ledgerStorageField.set(bookieServer.getBookie(), spyLedgerStorage);
+
+        HttpEndpointService triggerEntryLocationCompactService = bkHttpServiceProvider
+                .provideHttpEndpointService(HttpServer.ApiType.TRIGGER_ENTRY_LOCATION_COMPACT);
+
+        // 1. Put
+        // 1.1 Trigger all entry location rocksDB compact, should return OK
+        HttpServiceRequest request1 = new HttpServiceRequest("{\"entryLocationRocksDBCompact\":true}",
+                HttpServer.Method.PUT, null);
+        HttpServiceResponse response1 = triggerEntryLocationCompactService.handle(request1);
+        assertEquals(HttpServer.StatusCode.OK.getValue(), response1.getStatusCode());
+        LOG.info("Get response: {}", response1.getBody());
+
+        // 1.2 Specified trigger entry location rocksDB compact, should return OK
+        String body2 = "{\"entryLocationRocksDBCompact\":true,\"entryLocations\""
+               + ":\"/data1/bookkeeper/ledgers/current/locations\"}";
+        HttpServiceRequest request2 = new HttpServiceRequest(body2, HttpServer.Method.PUT, null);
+        HttpServiceResponse response2 = triggerEntryLocationCompactService.handle(request2);
+        assertEquals(HttpServer.StatusCode.OK.getValue(), response2.getStatusCode());
+        LOG.info("Get response: {}", response2.getBody());
+        assertTrue(response2.getBody().contains("Triggered entry Location RocksDB"));
+
+        // 1.3 Specified invalid entry location rocksDB compact, should return BAD_REQUEST
+        String body3 = "{\"entryLocationRocksDBCompact\":true,\"entryLocations\""
+                + ":\"/invalid1/locations,/data2/bookkeeper/ledgers/current/locations\"}";
+        HttpServiceRequest request3 = new HttpServiceRequest(body3, HttpServer.Method.PUT, null);
+        HttpServiceResponse response3 = triggerEntryLocationCompactService.handle(request3);
+        assertEquals(HttpServer.StatusCode.BAD_REQUEST.getValue(), response3.getStatusCode());
+        LOG.info("Get response: {}", response3.getBody());
+        assertTrue(response3.getBody().contains("is invalid"));
+
+        // 1.4 Some rocksDB is running compact, should return OK
+        String body4 = "{\"entryLocationRocksDBCompact\":true,\"entryLocations\""
+                + ":\"/data1/bookkeeper/ledgers/current/locations,/data2/bookkeeper/ledgers/current/locations\"}";
+        HttpServiceRequest request4 = new HttpServiceRequest(body4, HttpServer.Method.PUT, null);
+        HttpServiceResponse response4 = triggerEntryLocationCompactService.handle(request4);
+        assertEquals(HttpServer.StatusCode.OK.getValue(), response4.getStatusCode());
+        LOG.info("Get response: {}", response4.getBody());
+
+        // 1.5 Put, empty body, should return BAD_REQUEST
+        HttpServiceRequest request5 = new HttpServiceRequest(null, HttpServer.Method.PUT, null);
+        HttpServiceResponse response5 = triggerEntryLocationCompactService.handle(request5);
+        assertEquals(HttpServer.StatusCode.BAD_REQUEST.getValue(), response5.getStatusCode());
+        LOG.info("Get response: {}", response5.getBody());
+
+        // 2. GET, should return OK
+        HttpServiceRequest request6 = new HttpServiceRequest(null, HttpServer.Method.GET, null);
+        HttpServiceResponse response6 = triggerEntryLocationCompactService.handle(request6);
+        assertEquals(HttpServer.StatusCode.OK.getValue(), response6.getStatusCode());
+        assertTrue(response6.getBody().contains("\"/data2/bookkeeper/ledgers/current/locations\" : true"));
+        assertTrue(response6.getBody().contains("\"/data1/bookkeeper/ledgers/current/locations\" : false"));
+
+        // 3. POST, should return NOT_FOUND
+        HttpServiceRequest request7 = new HttpServiceRequest(null, HttpServer.Method.POST, null);
+        HttpServiceResponse response7 = triggerEntryLocationCompactService.handle(request7);
+        assertEquals(HttpServer.StatusCode.METHOD_NOT_ALLOWED.getValue(), response7.getStatusCode());
     }
 }
