@@ -1,4 +1,4 @@
-/**
+/*
  * Licensed to the Apache Software Foundation (ASF) under one
  * or more contributor license agreements.  See the NOTICE file
  * distributed with this work for additional information
@@ -18,14 +18,12 @@
 package org.apache.distributedlog.bk;
 
 import com.google.common.collect.Lists;
-
 import java.io.IOException;
 import java.util.LinkedList;
 import java.util.List;
 import java.util.concurrent.CompletableFuture;
 import java.util.concurrent.CompletionStage;
 import java.util.function.Function;
-
 import org.apache.bookkeeper.client.LedgerHandle;
 import org.apache.bookkeeper.common.concurrent.FutureEventListener;
 import org.apache.bookkeeper.common.concurrent.FutureUtils;
@@ -529,7 +527,10 @@ public class SimpleLedgerAllocator implements LedgerAllocator, FutureEventListen
 
                     @Override
                     public void onFailure(Throwable cause) {
-                        LOG.debug("Fail to obtain the allocated ledger handle when closing the allocator : ", cause);
+                        if (LOG.isDebugEnabled()) {
+                            LOG.debug("Fail to obtain the allocated ledger handle when closing the allocator : "
+                                    , cause);
+                        }
                         FutureUtils.complete(closePromise, null);
                     }
                 });
@@ -537,7 +538,10 @@ public class SimpleLedgerAllocator implements LedgerAllocator, FutureEventListen
 
             @Override
             public void onFailure(Throwable cause) {
-                LOG.debug("Fail to obtain the allocated ledger handle when closing the allocator : ", cause);
+                if (LOG.isDebugEnabled()) {
+                    LOG.debug("Fail to obtain the allocated ledger handle when closing the allocator : "
+                            , cause);
+                }
                 FutureUtils.complete(closePromise, null);
             }
         });
